"use strict";
var path_1 = require('path');
var helpers = require('./helpers');
var Handlebars = require('handlebars');
var fs_extra_1 = require('fs-extra');
var watchr = require('watchr');
var helpers_1 = require('./helpers');
exports.safe = helpers_1.safe;
Handlebars.registerHelper(helpers);
var defaultAutogenHeader = "/*\n      ===============================================\n      |                                             |\n      |  AUTO GENERATED FILE! DO NOT EDIT DIRECTLY  |\n      |                                             |\n      ===============================================\n*/\n\n";
var defaultScaffoldingHeader = "/*\n      ===========================================================================================\n      |                                                                                         |\n      |  This file is a part of scaffolding. DO NOT move it, if you do it will be regenerated.  |\n      |                                                                                         |\n      ===========================================================================================\n*/\n\n";
function resolve() {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i - 0] = arguments[_i];
    }
    return path_1.resolve.apply(void 0, paths);
}
exports.resolve = resolve;
function loadFile(absolutePath) {
    return fs_extra_1.readFileSync(absolutePath, 'utf8');
}
exports.loadFile = loadFile;
function loadTemplate(absolutePath) {
    return Handlebars.compile(loadFile(absolutePath));
}
exports.loadTemplate = loadTemplate;
function registerPartial(name, content) {
    Handlebars.registerPartial(name, content);
}
exports.registerPartial = registerPartial;
function registerHelper(name, helper) {
    Handlebars.registerHelper(name, helper);
}
exports.registerHelper = registerHelper;
function loadPartial(name, absolutePath) {
    registerPartial(name, loadFile(absolutePath));
}
exports.loadPartial = loadPartial;
function loadScript(absolutePath) {
    delete require.cache[absolutePath];
    return require(absolutePath);
}
exports.loadScript = loadScript;
function writeAutogenerated(absolutePath, content, header) {
    if (header === void 0) { header = defaultAutogenHeader; }
    fs_extra_1.outputFileSync(absolutePath, header + content);
}
exports.writeAutogenerated = writeAutogenerated;
function writeScaffolded(absolutePath, content, overwrite, header) {
    if (overwrite === void 0) { overwrite = false; }
    if (header === void 0) { header = defaultScaffoldingHeader; }
    fs_extra_1.outputFileSync(absolutePath, header + content, { flag: overwrite ? 'w+' : 'wx+' });
}
exports.writeScaffolded = writeScaffolded;
function watch(paths, listener) {
    watchr.watch({
        paths: paths,
        listener: listener,
        catchupDelay: 200,
    });
}
exports.watch = watch;
