export interface I18nFormatter {
    (bundle: I18nBundle): string;
    <K extends string>(key: string): I18nStringBuilder<K, any>;
}
export interface I18nBundle {
    key: string;
    data?: any;
}
export interface I18nStringBuilder<K extends string, D> {
    (data?: D): string;
    key: K;
    path: string;
}
export interface Language {
    [key: string]: I18nStringBuilder<string, any>;
}
export declare function format(language: Language, fallback: Language | null, bundle: I18nBundle): string;
export declare function format(language: Language, fallback: Language | null, key: string): I18nStringBuilder<string, any>;
export declare function buildFormatter<F extends I18nFormatter>(language: Language, fallback?: Language | null): F;
export interface MessageFormatParameters {
    [param: string]: any;
}
export interface Compiler {
    constant(str: string): I18nStringBuilder<string, void>;
    callback<D>(callback: (data?: D) => string): I18nStringBuilder<string, D>;
    messageFormat<D extends MessageFormatParameters>(formatString: string): I18nStringBuilder<string, D>;
    describe<T>(tree: T): LanguageDescriptor<T>;
}
export interface LanguageDescriptor<T> {
    locale: string;
    tree: T;
}
export declare function buildCompiler(locale: string): Compiler;
export interface LanguageMap {
    [locale: string]: Language;
}
export interface Internationalizer<T> {
    mainTree: T;
    mainLocale: string;
    languages: LanguageMap;
    buildFormatter: <F extends I18nFormatter>(locale: string) => F;
}
export declare function internationalize<T>(mainLanguageDescriptor: LanguageDescriptor<T>, ...otherLanguageDescriptors: LanguageDescriptor<any>[]): Internationalizer<T>;
