"use strict";
function format(language, fallback, bundleOrKey) {
    if (process.env.NODE_ENV !== 'production' && !language) {
        throw new TypeError('language must be provided');
    }
    if (bundleOrKey === null) {
        return null;
    }
    if (typeof bundleOrKey === 'object') {
        var targetKey = language[bundleOrKey.key] || (fallback && fallback[bundleOrKey.key]);
        if (process.env.NODE_ENV !== 'production' && !targetKey) {
            throw new TypeError('provided bundle key does not exist in lanugage or the fallback');
        }
        return targetKey(bundleOrKey.data);
    }
    if (typeof bundleOrKey === 'string') {
        var targetKey = language[bundleOrKey] || (fallback && fallback[bundleOrKey]);
        if (process.env.NODE_ENV !== 'production' && !targetKey) {
            throw new TypeError('provided key does not exist in lanugage or the fallback');
        }
        return targetKey;
    }
    if (process.env.NODE_ENV !== 'production') {
        throw new TypeError('key or bundle object expected');
    }
}
exports.format = format;
function buildFormatter(language, fallback) {
    if (fallback === void 0) { fallback = null; }
    return (function (bundle) { return format(language, fallback, bundle); });
}
exports.buildFormatter = buildFormatter;
function buildCompiler(locale) {
    var MessageFormat = require('messageformat');
    var mf = new MessageFormat(locale);
    return {
        constant: function (str) {
            return (function () { return str; });
        },
        callback: function (cb) {
            return (function (d) { return cb(d); });
        },
        messageFormat: function (formatString) {
            var formatter = mf.compile(formatString);
            return (function (d) { return formatter(d); });
        },
        describe: function (tree) {
            return {
                locale: locale,
                tree: tree,
            };
        },
    };
}
exports.buildCompiler = buildCompiler;
function buildKey(prefix, key) {
    return "" + (prefix || '') + (prefix ? '_' : '') + key.toUpperCase();
}
function buildPath(prefix, key) {
    return "" + (prefix || '') + (prefix ? '.' : '') + key;
}
function processDescriptor(tree, language, prefix, path) {
    Object.keys(tree).forEach(function (key) {
        var processedKey = buildKey(prefix, key);
        var processedPath = buildPath(path, key);
        if (typeof tree[key] === 'function') {
            var stringBuilder = tree[key];
            stringBuilder.key = processedKey;
            stringBuilder.path = processedPath;
            language[processedKey] = stringBuilder;
        }
        else if (tree[key] && typeof tree[key] === 'object') {
            processDescriptor(tree[key], language, processedKey, processedPath);
        }
    });
}
function internationalize(mainLanguageDescriptor) {
    var otherLanguageDescriptors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        otherLanguageDescriptors[_i - 1] = arguments[_i];
    }
    var languages = {};
    var mainLanguage = {};
    processDescriptor(mainLanguageDescriptor.tree, mainLanguage);
    languages[mainLanguageDescriptor.locale] = mainLanguage;
    otherLanguageDescriptors.forEach(function (languageDescriptor) {
        var lang = {};
        processDescriptor(languageDescriptor.tree, lang);
        languages[languageDescriptor.locale] = lang;
    });
    var mainLocale = mainLanguageDescriptor.locale;
    return {
        languages: languages,
        mainLocale: mainLocale,
        mainTree: mainLanguageDescriptor.tree,
        buildFormatter: function (locale) {
            if (locale === mainLocale) {
                return buildFormatter(mainLanguage);
            }
            if (process.env.NODE_ENV !== 'production' && !languages[locale]) {
                throw new Error("The provided locale " + locale + " does not exist in the list of languages.");
            }
            return buildFormatter(languages[locale], mainLanguage);
        },
    };
}
exports.internationalize = internationalize;
