"use strict";
var chai_1 = require('chai');
var React = require('react');
var react_addons_test_utils_1 = require('react-addons-test-utils');
var pages_1 = require('../src/pages');
var navigator_1 = require('../src/navigator');
var Navigate_1 = require('../src/Navigate');
var component = function (props) {
    return React.createElement("input", {type: "number", value: String(props.state)});
};
var reducerBuilder = function () { return function (s, _a) {
    if (s === void 0) { s = 3; }
    var value = _a.value;
    return value ? value : s;
}; };
var tree = {
    'a': {
        contents: {
            'd': {
                contents: {
                    'e': {
                        titleKey: 'E',
                        component: component,
                        reducerBuilder: reducerBuilder,
                    },
                },
                titleKey: 'D',
            },
        },
        titleKey: 'A',
    },
};
var navigator = new navigator_1.Navigator(new pages_1.Pages(tree), 'ADMIN');
describe('Navigate', function () {
    it('should render nothing when no page is open', function () {
        var renderer = react_addons_test_utils_1.createRenderer();
        renderer.render(React.createElement(Navigate_1.Navigate, {navigator: navigator}));
        chai_1.expect(renderer.getRenderOutput()).to.be.null;
    });
    it('should render the active page and pass down the expecetd props', function () {
        var nav = navigator;
        var dispatch = function (a) { return nav = nav.reduce(a); };
        nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
        var renderer = react_addons_test_utils_1.createRenderer();
        renderer.render(React.createElement(Navigate_1.Navigate, {navigator: nav}));
        var output = renderer.getRenderOutput();
        chai_1.expect(output.type).to.be.equals(component);
        chai_1.expect(output.props.mutex).to.be.equals('1');
        chai_1.expect(output.props.path).to.be.equals('a/d/e');
        chai_1.expect(output.props.state).to.be.equals(3);
        chai_1.expect(output.props.title.key).to.be.equals('E');
        chai_1.expect(output.props.navigator).to.be.equals(nav);
    });
});
