"use strict";
var chai_1 = require('chai');
var pageInstance_1 = require('../src/pageInstance');
var desciptor = {
    component: function () { return null; },
    path: 'admin',
    reducerBuilder: function (_a) {
        var rank = _a.rank, fail = _a.fail;
        return function (s, a) {
            if (s === void 0) { s = { B: rank }; }
            if (!fail) {
                if (a.type === 'INC') {
                    return { B: rank + (a.payload || 0) };
                }
                return s;
            }
            return undefined;
        };
    },
    titleKey: 'ADMIN_TITLE',
};
var instance = new pageInstance_1.PageInstance(desciptor, {
    mutex: '1',
    path: 'admin',
    reducerBuilderData: { rank: 1 },
    titleData: { NAME: 'jim' },
});
describe('PageInstance', function () {
    it('should contain the correct mutex', function () {
        chai_1.expect(instance.mutex).to.be.equals('1');
    });
    it('should contain the correct path', function () {
        chai_1.expect(instance.path).to.be.equals('admin');
    });
    it('should contain the correct title', function () {
        chai_1.expect(instance.title).to.be.deep.equal({ data: { NAME: 'jim' }, key: 'ADMIN_TITLE' });
    });
    it('should throw if the reducer is faulty', function () {
        chai_1.expect(function () { return new pageInstance_1.PageInstance(desciptor, {
            mutex: '1',
            path: 'admin',
            reducerBuilderData: { fail: true, rank: 1 },
        }); }).to.throw();
    });
    it('should correctly calculate the initial state', function () {
        chai_1.expect(instance.state.B).to.be.equals(1);
    });
    it('should return the same object if state is unchanged after reduction', function () {
        var page2 = instance.reduce({ type: 'BLAH' });
        chai_1.expect(page2).to.be.equals(instance);
    });
    it('should return the another object if state is changed after reduction', function () {
        var page2 = instance.reduce({ payload: 2, type: 'INC' });
        chai_1.expect(page2).not.to.be.equals(instance);
        chai_1.expect(page2.state.B).to.be.equals(3);
    });
});
