"use strict";
var chai_1 = require('chai');
var pages_1 = require('../src/pages');
var component = function () { return null; };
var reducerBuilder = function () { return function () { return 'state!'; }; };
var tree = {
    'a': {
        contents: {
            'b': {
                contents: {
                    'c': {
                        titleKey: 'C',
                        component: component,
                        reducerBuilder: reducerBuilder,
                    },
                },
                hidden: true,
                titleKey: 'B',
            },
            'd': {
                contents: {
                    'e': {
                        component: component,
                        reducerBuilder: reducerBuilder,
                        hidden: true,
                        titleKey: 'E',
                    },
                },
                titleKey: 'D',
            },
        },
        titleKey: 'A',
    },
    'f': {
        component: component,
        reducerBuilder: reducerBuilder,
        titleKey: 'F',
    },
};
describe('buildPage', function () {
    it('should properly build a page', function () {
        var page = pages_1.buildPage(component, reducerBuilder, 'A', true);
        chai_1.expect(page.component).to.be.equals(component);
        chai_1.expect(page.reducerBuilder).to.be.equals(reducerBuilder);
        chai_1.expect(page.titleKey).to.be.equals('A');
        chai_1.expect(page.hidden).to.be.equals(true);
        chai_1.expect(page.persistent).to.be.equals(false);
    });
});
describe('Pages', function () {
    it('should correctly determine whether is has a folder of specified path', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(pages.hasFolder('a/d')).to.be.true;
    });
    it('should correctly determine whether is has a page of specified path', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(pages.hasPage('a/d/e')).to.be.true;
    });
    it('should correctly determine whether the specified path is hidden', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(pages.isHidden('a/d/e')).to.be.true;
        chai_1.expect(pages.isHidden('a/b')).to.be.true;
        chai_1.expect(pages.isHidden('a/b/c')).to.be.true;
    });
    it('should return a folder of specified path', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(pages.getFolder('a/d').titleKey).to.be.equals('D');
    });
    it('should return a page of specified path', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(pages.getPage('a/b/c').titleKey).to.be.equals('C');
    });
    it('should build valid page instances with unique ids', function () {
        var pages = new pages_1.Pages(tree);
        var ins1 = pages.buildInstance({ mutex: '1', path: 'a/b/c' });
        var ins2 = pages.buildInstance({ mutex: '2', path: 'a/d/e' });
        var ins3 = pages.buildInstance({ mutex: '3', path: 'f' });
        chai_1.expect(ins1.descriptor.path).to.be.equals('a/b/c');
        chai_1.expect(ins2.descriptor.path).to.be.equals('a/d/e');
        chai_1.expect(ins3.descriptor.path).to.be.equals('f');
        chai_1.expect(ins1.mutex).to.be.equals('1');
        chai_1.expect(ins2.mutex).to.be.equals('2');
        chai_1.expect(ins3.mutex).to.be.equals('3');
    });
    it('should fail to build a page from an unknown path', function () {
        var pages = new pages_1.Pages(tree);
        chai_1.expect(function () { return pages.buildInstance({ mutex: '1', path: 'blah!' }); }).to.throw();
    });
});
