"use strict";
var chai_1 = require('chai');
var persist_1 = require('../src/persist');
describe('persist', function () {
    describe('save, load', function () {
        it('should save and load correctly', function () {
            var home = { mutex: '1', path: 'home' };
            persist_1.save('', [home]);
            var admin = { mutex: '2', path: 'admin' };
            persist_1.save('ADM', [admin]);
            chai_1.expect(persist_1.load('')).to.deep.equal([home]);
            chai_1.expect(persist_1.load('ADM')).to.deep.equal([admin]);
        });
    });
    describe('save, load active page', function () {
        it('should save and load the active page correctly', function () {
            persist_1.saveActivePage('', 'home');
            persist_1.saveActivePage('ADM', 'admin');
            chai_1.expect(persist_1.loadActivePage('')).to.be.equals('home');
            chai_1.expect(persist_1.loadActivePage('ADM')).to.be.equals('admin');
        });
    });
    describe('clear', function () {
        it('should clear pages and the active page without affecting other values', function () {
            var home = { mutex: '1', path: 'home' };
            persist_1.save('', [home]);
            persist_1.saveActivePage('', 'home');
            persist_1.saveActivePage('ADM', 'admin');
            persist_1.clear('');
            chai_1.expect(persist_1.load('')).to.be.deep.equal([]);
            chai_1.expect(persist_1.loadActivePage('')).to.be.null;
            chai_1.expect(persist_1.loadActivePage('ADM')).to.be.equals('admin');
        });
    });
});
