/// <reference types="bluebird" />
import * as Bluebird from 'bluebird';
export declare type ValidTargetType = number | string | symbol;
export interface Dispatch {
    (action: Action<any, any>): void;
    (thunk: Thunk): Bluebird<void>;
}
export interface Thunk {
    (dispatch: Dispatch): Bluebird<void>;
}
export interface Action<TPayload, TMeta> {
    type: string;
    payload?: TPayload;
    meta?: TMeta;
}
export interface ActionTransformer {
    <A extends Action<any, any>>(action: A): A;
}
export interface TargetedAction<TPayload, TMeta> extends Action<TPayload, TMeta> {
    target?: ValidTargetType | ValidTargetType[];
    broadcast: boolean;
}
export interface ActionDescriptor<TPayload, TMeta> {
    type: string;
    create(payload?: TPayload, meta?: TMeta): Action<TPayload, TMeta>;
    is(action: Action<any, any>): action is Action<TPayload, TMeta>;
}
export declare function composeTransformers<A extends Action<any, any>>(transformers?: ActionTransformer | ActionTransformer[]): ActionTransformer;
export declare function createActionDescriptor<TPayload, TMeta>(type: string, transformers?: ActionTransformer | ActionTransformer[]): ActionDescriptor<TPayload, TMeta>;
export interface TargetedActionDescriptorCreator<TPayload, TMeta> {
    unicast(target: ValidTargetType, payload?: TPayload, meta?: TMeta): TargetedAction<TPayload, TMeta>;
    multicast(targets: ValidTargetType[], payload?: TPayload, meta?: TMeta): TargetedAction<TPayload, TMeta>;
    broadcast(payload?: TPayload, meta?: TMeta): TargetedAction<TPayload, TMeta>;
}
export interface TargetedActionDescriptor<TPayload, TMeta> {
    type: string;
    create: TargetedActionDescriptorCreator<TPayload, TMeta>;
    is(id: ValidTargetType, action: Action<any, any>): action is TargetedAction<TPayload, TMeta>;
}
export declare function createTargetedActionDescriptor<TPayload, TMeta>(type: string, transformers?: ActionTransformer | ActionTransformer[]): TargetedActionDescriptor<TPayload, TMeta>;
export interface AsyncActionDescriptor<TPayload> {
    create: (payload?: TPayload, meta?: any) => Thunk;
}
export declare function createAsyncAction<TPayload, TResult>(resolve: (data?: TPayload) => Bluebird<TResult>, success: ActionDescriptor<TResult, any>, fail?: ActionDescriptor<any, any>, attempt?: ActionDescriptor<TPayload, any>): AsyncActionDescriptor<TPayload>;
export declare function hook(dispatch: Dispatch, ...transformers: ActionTransformer[]): Dispatch;
