"use strict";
var chai_1 = require('chai');
var emission_1 = require('./emission');
describe('emissionMiddleware', function () {
    it('should append dispatch and getState', function () {
        var action;
        var dispatch = function (a) { return action = a; };
        var getState = function () { return ({ state: 'hello' }); };
        emission_1.emissionMiddleware({ dispatch: dispatch, getState: getState })(dispatch)({ type: 'TYPE' });
        chai_1.expect(action.type).to.be.equals('TYPE');
        chai_1.expect(action[emission_1.EMISSION_DISPATCH]).to.be.equals(dispatch);
        chai_1.expect(action[emission_1.EMISSION_GET_STATE]).to.be.equals(getState);
    });
});
describe('emit', function () {
    it('should properly defer execution of side-effect', function () {
        var action;
        var dispatch = function (a) { return action = a; };
        var getState = function () { return ({ state: 'HELLO' }); };
        emission_1.emissionMiddleware({ dispatch: dispatch, getState: getState })(dispatch)({ type: 'TYPE' });
        var promise = emission_1.emit(action, function (d, gS) {
            d(action = { type: gS().state });
            return 'stuff';
        });
        chai_1.expect(action.type).to.be.equals('TYPE');
        return promise.then(function (stuff) {
            chai_1.expect(stuff).to.be.equals('stuff');
            chai_1.expect(action.type).to.be.equals('HELLO');
        });
    });
});
