import { Action } from './action';
export interface Reducer<TState> {
    (state: TState | undefined, action: Action<any, any>): TState;
}
export interface ReducerMap {
    [reducer: string]: Reducer<any>;
}
export interface Reducible {
    reduce(action: Action<any, any>): this;
}
export interface AutoReducibleState {
    [substate: string]: Reducible;
}
export interface Setter<TProps> {
    (prop: TProps, value: any): void;
}
export interface Unsetter<TProps> {
    (prop: TProps): void;
}
export interface Resetter<TState> {
    (state?: TState): void;
}
export interface Mutator<TState, TProps extends string> {
    (state: TState, action: Action<any, any>, set: Setter<TProps>, unset: Unsetter<TProps>, reset: Resetter<TState>): void;
}
export declare function createMutatorReducer<TState, TProps extends string>(initialState: TState, mutator: Mutator<TState, TProps>): Reducer<TState>;
export declare function createAutoReducer<TState extends AutoReducibleState>(initialState: TState): Reducer<TState>;
export declare function createReducibleReducer<T extends Reducible>(reducible: T): Reducer<T>;
export interface Constant<T> extends Reducible {
    value: T;
}
export declare function createConstantReducer<T>(value: T): Constant<T>;
export declare const combineReducers: <TState>(map: ReducerMap) => Reducer<TState>;
