"use strict";
var redux_1 = require('redux');
var lodash_1 = require('lodash');
function createMutatorReducer(initialState, mutator) {
    return function reducer(state, action) {
        if (state === void 0) { state = initialState; }
        var modified = false;
        var set = null;
        var unset = null;
        var reset = null;
        function setter(prop, value) {
            if (value !== state[prop]) {
                modified = true;
                if (!set) {
                    set = {};
                }
                set[prop] = value;
                if (unset) {
                    var unsetIndex = unset.indexOf(prop);
                    if (unsetIndex !== -1) {
                        unset.splice(unsetIndex, 1);
                    }
                }
            }
        }
        function unsetter(prop) {
            if (state.hasOwnProperty(prop)) {
                modified = true;
                if (!unset) {
                    unset = [];
                }
                unset.push(prop);
            }
        }
        function resetter(resetState) {
            if (resetState) {
                modified = true;
                reset = resetState;
            }
            else if (state !== initialState) {
                modified = true;
                reset = initialState;
            }
        }
        mutator(state, action, setter, unsetter, resetter);
        if (!modified) {
            return state;
        }
        if (reset) {
            return reset;
        }
        var next = lodash_1.assign({}, state, set);
        if (unset) {
            unset.forEach(function (prop) { return delete next[prop]; });
        }
        return next;
    };
}
exports.createMutatorReducer = createMutatorReducer;
function createAutoReducer(initialState) {
    var properties = Object.keys(initialState);
    var initializationPart = '';
    var modificationCheckPart = '';
    var returnPart = '';
    properties.forEach(function (property) {
        if (typeof initialState[property].reduce !== 'function') {
            throw new Error("initialState has a property (" + property + ") which is not a Reducible.");
        }
        initializationPart += "var " + property + " = state." + property + ".reduce(action);\n";
        modificationCheckPart += "|| " + property + " !== state." + property + " ";
        returnPart += property + ": " + property + ",\n";
    });
    var body = "\n    " + initializationPart + "\n\n    if (false " + modificationCheckPart + ") {\n      return {\n        " + returnPart + "\n      };\n    }\n\n    return state;\n  ";
    var reducer = Function('state', 'action', body);
    return function (state, action) {
        if (state === void 0) { state = initialState; }
        return reducer(state, action);
    };
}
exports.createAutoReducer = createAutoReducer;
function createReducibleReducer(reducible) {
    return function (state, action) {
        if (state === void 0) { state = reducible; }
        return state.reduce(action);
    };
}
exports.createReducibleReducer = createReducibleReducer;
function createConstantReducer(value) {
    var constantReducer = { value: value };
    constantReducer.reduce = function () { return constantReducer; };
    return constantReducer;
}
exports.createConstantReducer = createConstantReducer;
exports.combineReducers = redux_1.combineReducers;
