"use strict";
var chai_1 = require('chai');
var Bluebird = require('bluebird');
var emission_1 = require('./emission');
var store_1 = require('./store');
describe('createStore', function () {
    it('should properly create normal store', function () {
        var obj = {};
        var store = store_1.createStore(function (s) {
            if (s === void 0) { s = obj; }
            return s;
        });
        chai_1.expect(store.getState()).to.be.equals(obj);
    });
    it('should properly create logger store', function () {
        var obj = {};
        var store = store_1.createStore(function (s) {
            if (s === void 0) { s = obj; }
            return s;
        }, true);
        chai_1.expect(store.getState()).to.be.equals(obj);
    });
    it('should properly create store with thunk support', function () {
        var obj = {};
        var obj2 = {};
        var store = store_1.createStore(function (s, _a) {
            if (s === void 0) { s = obj; }
            var type = _a.type;
            return type === 'T' ? obj2 : s;
        });
        return store.dispatch(function (dispatch) { return Bluebird.delay(0).then(function () { return dispatch({ type: 'T' }); }); }).then(function () { return chai_1.expect(store.getState()).to.be.equals(obj2); });
    });
    it('should properly create store with emission support', function () {
        var obj = {};
        var emitIsWorking = false;
        var promise = null;
        var store;
        var reducer = function (state, action) {
            if (state === void 0) { state = obj; }
            if (action.type === 'A') {
                chai_1.expect(action[emission_1.EMISSION_GET_STATE]).to.be.equals(store.getState);
                chai_1.expect(action[emission_1.EMISSION_DISPATCH]).to.be.a('function');
            }
            if (action.type === 'B') {
                promise = emission_1.emit(action, function (dispatch, getState) {
                    chai_1.expect(getState()).to.be.equals(obj);
                    dispatch({ type: 'A' });
                    dispatch({ type: 'C' });
                });
            }
            if (action.type === 'C') {
                emitIsWorking = true;
            }
            return state;
        };
        store = store_1.createStore(reducer);
        store.dispatch({ type: 'A' });
        store.dispatch({ type: 'B' });
        return promise.then(function () {
            chai_1.expect(emitIsWorking).to.be.true;
        });
    });
});
