"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
var lodash_1 = require('lodash');
var datatypes_1 = require('./datatypes');
__export(require('./datatypes'));
function getCommonDefaults(owner) {
    return {
        owner: owner,
        autoIncrement: false,
        mutable: true,
        name: null,
        primary: false,
        required: false,
        type: null,
        unique: false,
    };
}
function getPrimaryKeyDefaults(owner) {
    return lodash_1.assign(getCommonDefaults(owner), {
        autoIncrement: true,
        mutable: false,
        primary: true,
        type: datatypes_1.DataTypes.INTEGER(),
    });
}
function validateAttribute(attribute) {
    if (process.env.NODE_ENV !== 'production') {
        if (!attribute.name || typeof attribute.name !== 'string') {
            throw new Error("Attribute name < " + attribute.name + " > is invalid");
        }
        if (!(attribute.type instanceof datatypes_1.AbstractType)) {
            throw new Error("Attribute " + attribute.name + " does not have a valid type.");
        }
        if (attribute.autoIncrement && !attribute.primary) {
            throw new Error("Only primary keys can be auto increment. Check " + attribute.name + ".");
        }
        if (attribute.autoIncrement && !datatypes_1.IntegerType.is(attribute.type)) {
            throw new Error("Only integer attributes can be auto incremented. Check " + attribute.name + ".");
        }
        if (attribute.primary && (attribute.unique || attribute.required)) {
            throw new Error("primary means unique and required. Providing both is redundant.");
        }
        if (attribute.validator && typeof attribute.validator !== 'function') {
            throw new Error("Attribute " + attribute.name + " has non function validator.");
        }
        if (attribute.defaultValue !== undefined &&
            attribute.validator &&
            attribute.validator(attribute.defaultValue) !== null) {
            throw new Error("The validator does not validate the defaultValue of " + attribute.defaultValue);
        }
    }
}
exports.validateAttribute = validateAttribute;
function appendAttribute(model) {
    var attributes = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        attributes[_i - 1] = arguments[_i];
    }
    var attribute = lodash_1.assign.apply(void 0, [{}].concat(attributes));
    if (process.env.NODE_ENV !== 'production') {
        if (!model) {
            throw new Error('Model must be provided.');
        }
        if (attributes.length === 0) {
            throw new Error("No attribute provided to be appended to " + model.name + ".");
        }
        if (lodash_1.chain(model.attributes).map('name').includes(attribute.name).value()) {
            throw new Error("Attribute " + attribute.name + " of type " + attribute.type.typeName + " was added twice.");
        }
        if (attribute.primary) {
            var previousPrimary = lodash_1.find(model.attributes, function (a) { return a.primary; });
            if (previousPrimary) {
                throw new Error("Model " + model.name + " already has a primaryKey attribute of " + previousPrimary.name + ".");
            }
        }
        validateAttribute(attribute);
    }
    model.attributes.push(attribute);
}
exports.appendAttribute = appendAttribute;
function createAttributeBuilder(model) {
    if (process.env.NODE_ENV !== 'production' && !model) {
        throw new Error('model must be provided');
    }
    var commonDefaults = getCommonDefaults(model.owner);
    var pkDefaults = getPrimaryKeyDefaults(model.owner);
    function attributeBuilder(options) {
        appendAttribute(model, commonDefaults, options);
    }
    var ab = attributeBuilder;
    ab.primaryKey = function buildPrimaryKey(options) {
        if (options && datatypes_1.StringType.is(options.type) && !('autoIncrement' in options)) {
            options.autoIncrement = false;
        }
        appendAttribute(model, pkDefaults, options);
    };
    return ab;
}
exports.createAttributeBuilder = createAttributeBuilder;
