"use strict";
var chai_1 = require('chai');
var EnumType_1 = require('./EnumType');
describe('EnumType', function () {
    it('should correctly instantiate and identify an enum datatype', function () {
        var d = EnumType_1.default.create('A', [['a', 'A']]);
        chai_1.expect(d.typeName).to.be.equals('string');
        chai_1.expect(EnumType_1.default.is(d)).to.be.true;
        chai_1.expect(d.values[0].name).to.be.equals('a');
        chai_1.expect(d.values[0].i18nKey).to.be.equals('A');
    });
    it('should properly override the typeName', function () {
        var d = EnumType_1.default.create('A', [['a', 'A']], "'a'");
        chai_1.expect(d.typeName).to.be.equals("'a'");
    });
    it('should throw if no values are specified', function () {
        chai_1.expect(function () { return EnumType_1.default.create('A', [], "'a'"); }).to.throw();
        chai_1.expect(function () { return EnumType_1.default.create('A', [0], "'a'"); }).to.throw();
        chai_1.expect(function () { return EnumType_1.default.create('A', [false], "'a'"); }).to.throw();
        chai_1.expect(function () { return EnumType_1.default.create('A', [null], "'a'"); }).to.throw();
        chai_1.expect(function () { return EnumType_1.default.create('A', [undefined], "'a'"); }).to.throw();
    });
    it('should throw if any of the values have invalid name', function () {
        chai_1.expect(function () { return EnumType_1.default.create('A', [[null, 'A'], ['a', 'A']], "'a'"); }).to.throw();
    });
    it('should throw if any of the values have the same name', function () {
        chai_1.expect(function () { return EnumType_1.default.create('A', [['a', 'A'], ['a', 'A']], "'a'"); }).to.throw();
        chai_1.expect(function () { return EnumType_1.default.create('A', [['a', 'A'], ['a', 'B']], "'a'"); }).to.throw();
    });
});
