"use strict";
var chai_1 = require('chai');
var RangeType_1 = require('./RangeType');
var index_1 = require('./index');
describe('RangeType', function () {
    it('should correctly instantiate and identify a range datatype', function () {
        var d = RangeType_1.default.create(index_1.DataTypes.DATE());
        chai_1.expect(d.typeName).to.be.equals('[Date, Date]');
        chai_1.expect(RangeType_1.default.is(d)).to.be.true;
    });
    it('should properly override the typeName', function () {
        var d = RangeType_1.default.create(index_1.DataTypes.DATE(), 'any[]');
        chai_1.expect(d.typeName).to.be.equals('any[]');
    });
    it('should throw if subtype is not valid', function () {
        chai_1.expect(function () { return RangeType_1.default.create(null); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create({}); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(true); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create([]); }).to.throw();
    });
    it('should throw if subtype is not a valid range', function () {
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.ENUM('a', [['a', 'A']])); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.BOOLEAN()); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.DOUBLE()); }).not.to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.INTEGER()); }).not.to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.STRING()); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.OBJECT()); }).to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.DATE()); }).not.to.throw();
        chai_1.expect(function () { return RangeType_1.default.create(index_1.DataTypes.RANGE(index_1.DataTypes.STRING())); }).to.throw();
    });
});
