"use strict";
var lodash_1 = require('lodash');
var relation_1 = require('./relation');
var spec_1 = require('./spec');
var attribute_1 = require('./attribute');
function joinee(model, aliasOnJoin, aliasOnTarget, options) {
    if (process.env.NODE_ENV !== 'production') {
        if (!model) {
            throw new Error('model is not provided');
        }
        if (typeof aliasOnJoin !== 'string') {
            throw new Error("alias " + aliasOnJoin + " is not valid");
        }
        if (typeof aliasOnTarget !== 'string') {
            throw new Error("alias " + aliasOnTarget + " is not valid");
        }
    }
    return {
        model: model,
        aliasOnJoin: aliasOnJoin,
        aliasOnTarget: aliasOnTarget,
        key: (options && options.key) || aliasOnJoin + "Id",
        onDelete: (options && options.onDelete) || 'CASCADE',
        onUpdate: (options && options.onUpdate) || 'CASCADE',
        type: (options && options.type) || attribute_1.DataTypes.INTEGER(),
    };
}
exports.joinee = joinee;
var defaultCreateJoinModelOptions = {
    firstJoinee: null,
    name: null,
    owner: null,
    secondJoinee: null,
    virtual: false,
};
function createJoinModel(options, initializer) {
    var normalizedOptions = lodash_1.assign({}, defaultCreateJoinModelOptions, options);
    var firstJoinee = normalizedOptions.firstJoinee;
    var secondJoinee = normalizedOptions.secondJoinee;
    var model = {
        attributes: [],
        indexes: {
            unique: [[firstJoinee.key, secondJoinee.key]],
        },
        join: true,
        name: normalizedOptions.name,
        owner: normalizedOptions.owner,
        relations: [],
        virtual: normalizedOptions.virtual,
    };
    var builder = {
        attribute: attribute_1.createAttributeBuilder(model),
    };
    builder.attribute({
        name: firstJoinee.key,
        required: true,
        type: firstJoinee.type,
    });
    builder.attribute({
        name: secondJoinee.key,
        required: true,
        type: secondJoinee.type,
    });
    relation_1.appendRelation(model, {
        alias: firstJoinee.aliasOnJoin,
        key: firstJoinee.key,
        model: firstJoinee.model.name,
        onDelete: firstJoinee.onDelete,
        onUpdate: firstJoinee.onUpdate,
        owner: normalizedOptions.owner,
        required: true,
        type: 'MASTER',
        virtual: firstJoinee.model.virtual,
    });
    relation_1.appendRelation(model, {
        alias: secondJoinee.aliasOnJoin,
        key: secondJoinee.key,
        model: secondJoinee.model.name,
        onDelete: secondJoinee.onDelete,
        onUpdate: secondJoinee.onUpdate,
        owner: normalizedOptions.owner,
        required: true,
        type: 'MASTER',
        virtual: secondJoinee.model.virtual,
    });
    relation_1.appendRelation(firstJoinee.model.model, {
        alias: secondJoinee.aliasOnTarget,
        key: firstJoinee.key,
        model: secondJoinee.model.name,
        onDelete: firstJoinee.onDelete,
        onUpdate: firstJoinee.onUpdate,
        owner: normalizedOptions.owner,
        required: true,
        through: model.name,
        type: 'MANY_TO_MANY',
        virtual: normalizedOptions.virtual,
    });
    relation_1.appendRelation(secondJoinee.model.model, {
        alias: firstJoinee.aliasOnTarget,
        key: secondJoinee.key,
        model: firstJoinee.model.name,
        onDelete: secondJoinee.onDelete,
        onUpdate: secondJoinee.onUpdate,
        owner: normalizedOptions.owner,
        required: true,
        through: model.name,
        type: 'MANY_TO_MANY',
        virtual: normalizedOptions.virtual,
    });
    function initialize() {
        if (typeof initializer === 'function') {
            initializer(builder);
        }
        return new spec_1.Spec(model);
    }
    return {
        model: model,
        join: true,
        name: normalizedOptions.name,
        owner: normalizedOptions.owner,
        virtual: normalizedOptions.virtual,
        initialize: initialize,
    };
}
exports.createJoinModel = createJoinModel;
