"use strict";
var attribute_1 = require('./attribute');
var relation_1 = require('./relation');
var spec_1 = require('./spec');
function createModel(options, initializer) {
    var model = {
        attributes: [],
        indexes: { unique: [] },
        join: false,
        name: options.name,
        owner: options.owner,
        relations: [],
        virtual: !!options.virtual,
    };
    var builder = {
        attribute: attribute_1.createAttributeBuilder(model),
        relation: relation_1.createRelationBuilder(model),
        validator: function (validator) {
            model.validator = validator;
        },
        multicolumnUniqueIndex: function () {
            var fields = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fields[_i - 0] = arguments[_i];
            }
            model.indexes.unique.push(fields);
        },
    };
    function initialize() {
        if (typeof initializer === 'function') {
            initializer(builder);
        }
        return new spec_1.Spec(model);
    }
    return {
        initialize: initialize,
        model: model,
        join: model.join,
        name: model.name,
        owner: model.owner,
        virtual: model.virtual,
    };
}
exports.createModel = createModel;
