"use strict";
var chai_1 = require('chai');
var model_1 = require('./model');
describe('model', function () {
    describe('createModel', function () {
        it('should properly build an UninitializedStateModel object', function () {
            var model = model_1.createModel({
                name: 'a',
                owner: ['b', 'c'],
                virtual: false,
            });
            chai_1.expect(model.join).to.be.false;
            chai_1.expect(model.owner).to.deep.equal(['b', 'c']);
            chai_1.expect(model.name).to.be.equals('a');
            chai_1.expect(model.virtual).to.be.false;
            chai_1.expect(model.initialize).to.be.a('function');
        });
        it('should properly call the builder with valid attribute and relation builder', function () {
            var model = model_1.createModel({ name: 'a', owner: null }, function (_a) {
                var attribute = _a.attribute, relation = _a.relation;
                chai_1.expect(attribute).to.be.a('function');
                chai_1.expect(attribute.primaryKey).to.be.a('function');
                chai_1.expect(relation.inheritors).to.be.a('function');
                chai_1.expect(relation.injective).to.be.a('function');
                chai_1.expect(relation.master).to.be.a('function');
                chai_1.expect(relation.recursive).to.be.a('function');
            });
            model.initialize();
        });
        it('should properly call the builder with valid multicolumnUniqueIndex builder', function () {
            var model = model_1.createModel({ name: 'a', owner: null }, function (_a) {
                var multicolumnUniqueIndex = _a.multicolumnUniqueIndex;
                chai_1.expect(multicolumnUniqueIndex).to.be.a('function');
                multicolumnUniqueIndex('a', 'b');
                multicolumnUniqueIndex('c', 'd');
                multicolumnUniqueIndex('e', 'f', 'g');
            });
            model.initialize();
            chai_1.expect(model.model.indexes.unique[0]).to.deep.equal(['a', 'b']);
            chai_1.expect(model.model.indexes.unique[1]).to.deep.equal(['c', 'd']);
            chai_1.expect(model.model.indexes.unique[2]).to.deep.equal(['e', 'f', 'g']);
        });
        it('should properly call the builder with valid validator setter', function () {
            var noop = function () { return null; };
            var model = model_1.createModel({ name: 'a', owner: null }, function (_a) {
                var validator = _a.validator;
                chai_1.expect(validator).to.be.a('function');
                validator(noop);
            });
            model.initialize();
            chai_1.expect(model.model.validator).to.be.equals(noop);
        });
    });
});
