import { Validator } from 'pretur.validation';
import { AbstractType } from './attribute';
import { Model, UninitializedStateModel, Owner } from './model';
export declare type ModificationActions = 'RESTRICT' | 'CASCADE' | 'NO ACTION' | 'SET NULL' | 'SET DEFAULT';
export declare type RelationType = 'SUPERCLASS' | 'SUBCLASS' | 'MASTER' | 'DETAIL' | 'RECURSIVE' | 'MANY_TO_MANY' | 'INJECTIVE';
export interface Relation {
    type: RelationType;
    model: string;
    owner?: Owner;
    alias: string;
    key: string;
    through?: string;
    required?: boolean;
    virtual?: boolean;
    onDelete: ModificationActions;
    onUpdate: ModificationActions;
}
export interface Inheritor {
    target: UninitializedStateModel<any>;
    alias: string;
    i18nKey: string;
}
export interface InheritorsOptions<T> {
    sharedExistingUniqueField: string;
    aliasOnSubclasses: string;
    typeIdentifierFieldName?: string;
    typeIdentifierEnumTypeName?: string;
    typeIdentifierRequired?: boolean;
    typeIdentifierValidator?: Validator<T>;
    inheritors: Inheritor[];
}
export interface MasterOptions<T> {
    target: UninitializedStateModel<any>;
    alias: string;
    ownAliasOnTarget: string;
    foreignKey?: string;
    foreignKeyType?: AbstractType;
    required?: boolean;
    onDelete?: ModificationActions;
    onUpdate?: ModificationActions;
    validator?: Validator<T>;
    owner?: Owner;
    targetOwner?: Owner;
}
export interface InjectiveOptions<T> {
    target: UninitializedStateModel<any>;
    alias: string;
    ownAliasOnTarget: string;
    foreignKey?: string;
    foreignKeyType?: AbstractType;
    required?: boolean;
    onDelete?: ModificationActions;
    onUpdate?: ModificationActions;
    validator?: Validator<T>;
    owner?: Owner;
    targetOwner?: Owner;
}
export interface RecursiveOptions<T> {
    alias: string;
    key?: string;
    keyType?: AbstractType;
    onDelete?: ModificationActions;
    onUpdate?: ModificationActions;
    validator?: Validator<T>;
}
export declare function appendRelation(model: Model<any>, ...relations: Relation[]): void;
export interface RelationsBuilder {
    inheritors<T>(options: InheritorsOptions<T>): void;
    master<T>(options: MasterOptions<T>): void;
    injective<T>(options: InjectiveOptions<T>): void;
    recursive<T>(options: RecursiveOptions<T>): void;
}
export declare function createRelationBuilder(model: Model<any>): RelationsBuilder;
