"use strict";
var lodash_1 = require('lodash');
function ownersIntersect(first, second) {
    var f = lodash_1.castArray(first);
    var s = lodash_1.castArray(second);
    return lodash_1.intersection(f, s).filter(function (i) { return i; }).length > 0;
}
function populateRelations(relationsObj, relationsArray) {
    relationsArray.forEach(function (relation) {
        switch (relation.type) {
            case 'SUPERCLASS':
                relationsObj.superclass.push(relation);
                break;
            case 'SUBCLASS':
                relationsObj.subclass.push(relation);
                break;
            case 'MASTER':
                relationsObj.master.push(relation);
                break;
            case 'DETAIL':
                relationsObj.detail.push(relation);
                break;
            case 'RECURSIVE':
                relationsObj.recursive.push(relation);
                break;
            case 'MANY_TO_MANY':
                relationsObj.manyToMany.push(relation);
                break;
            case 'INJECTIVE':
                relationsObj.injective.push(relation);
                break;
        }
    });
}
var Spec = (function () {
    function Spec(model) {
        var _this = this;
        this.byAlias = function (alias) { return lodash_1.find(_this.model.relations, { alias: alias }); };
        this.nonVirtualByAlias = function (alias) {
            return lodash_1.find(_this.model.relations.filter(function (r) { return !r.virtual; }), { alias: alias });
        };
        this.model = model;
    }
    Object.defineProperty(Spec.prototype, "name", {
        get: function () {
            return this.model.name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "owner", {
        get: function () {
            return this.model.owner;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "virtual", {
        get: function () {
            return this.model.virtual;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "join", {
        get: function () {
            return this.model.join;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "validator", {
        get: function () {
            return this.model.validator;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "attributes", {
        get: function () {
            var map = {};
            this.model.attributes.forEach(function (attrib) { return map[attrib.name] = attrib; });
            return map;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "attributeArray", {
        get: function () {
            return this.model.attributes;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "indexes", {
        get: function () {
            return this.model.indexes;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "relations", {
        get: function () {
            var rels = {
                byAlias: this.byAlias,
                detail: [],
                injective: [],
                manyToMany: [],
                master: [],
                recursive: [],
                subclass: [],
                superclass: [],
            };
            populateRelations(rels, this.model.relations);
            return rels;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "relationArray", {
        get: function () {
            return this.model.relations;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "nonVirtualRelations", {
        get: function () {
            var rels = {
                byAlias: this.nonVirtualByAlias,
                detail: [],
                injective: [],
                manyToMany: [],
                master: [],
                recursive: [],
                subclass: [],
                superclass: [],
            };
            var nonVirtualRelations = this.model.relations.filter(function (r) { return !r.virtual; });
            populateRelations(rels, nonVirtualRelations);
            return rels;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "nonVirtualRelationArray", {
        get: function () {
            return this.model.relations.filter(function (r) { return !r.virtual; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "dependencies", {
        get: function () {
            var allRelations = this.model.relations;
            return lodash_1.uniq(allRelations.map(function (r) { return r.model; }).concat(allRelations.filter(function (r) { return r.type === 'MANY_TO_MANY'; }).map(function (r) { return r.through; }))).sort();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Spec.prototype, "nonVirtualDependencies", {
        get: function () {
            var allRelations = this.model.relations.filter(function (r) { return !r.virtual; });
            return lodash_1.uniq(allRelations.map(function (r) { return r.model; }).concat(allRelations.filter(function (r) { return r.type === 'MANY_TO_MANY'; }).map(function (r) { return r.through; }))).sort();
        },
        enumerable: true,
        configurable: true
    });
    Spec.prototype.filterByOwner = function (owner) {
        if (!owner || !this.model.owner || owner.length === 0 || this.model.owner.length === 0) {
            return this;
        }
        if (!ownersIntersect(owner, this.model.owner)) {
            return null;
        }
        var newModel = lodash_1.assign({}, this.model, {
            attributes: this.model.attributes.filter(function (a) { return !a.owner || ownersIntersect(owner, a.owner); }),
            relations: this.model.relations.filter(function (r) { return !r.owner || ownersIntersect(owner, r.owner); }),
        });
        return new Spec(newModel);
    };
    return Spec;
}());
exports.Spec = Spec;
