"use strict";
var Bluebird = require('bluebird');
var fetch_1 = require('./fetch');
function buildFetcherCreator(endPointUrl) {
    function buildFetcher() {
        var id = 0;
        var feched = false;
        var requests = [];
        var listeners = [];
        function addId(query) {
            id += 1;
            query.queryId = id;
            return query;
        }
        function add(query) {
            return new Bluebird(function (resolve, reject) {
                requests.push({ query: addId(query), resolve: resolve, reject: reject });
            });
        }
        function listen() {
            return new Bluebird(function (resolve, reject) {
                listeners.push({ resolve: resolve, reject: reject });
            });
        }
        function fetch() {
            if (process.env.NODE_ENV !== 'production' && feched) {
                throw new Error('fetch cannot be called on the same object more than once');
            }
            if (requests.length === 0) {
                return Bluebird.resolve(false);
            }
            feched = true;
            return fetch_1.fetch({
                body: requests.map(function (r) { return r.query; }),
                method: 'POST',
                url: endPointUrl,
            }).then(function (response) {
                var requestsByQueryId = {};
                requests.forEach(function (req) { return requestsByQueryId[req.query.queryId] = req; });
                response.body.forEach(function (item) {
                    var request = requestsByQueryId[item.queryId];
                    request.resolve({
                        count: item.count,
                        data: item.data,
                        error: item.error,
                        ok: response.ok,
                        status: response.status,
                        statusText: response.statusText,
                    });
                });
                var errors = response.body.filter(function (item) { return item.error; }).map(function (item) { return item.error; });
                listeners.forEach(function (listener) { return listener.resolve({
                    errors: errors,
                    ok: response.ok,
                    status: response.status,
                    statusText: response.statusText,
                }); });
            }).catch(function (error) {
                requests.forEach(function (request) { return request.reject(error); });
                listeners.forEach(function (listener) { return listener.reject(error); });
            }).then(function () { return true; });
        }
        return { add: add, fetch: fetch, listen: listen };
    }
    function buildSingleShotFetcher(harness) {
        var fetcher = buildFetcher();
        harness(fetcher);
        return fetcher.fetch();
    }
    return { buildFetcher: buildFetcher, buildSingleShotFetcher: buildSingleShotFetcher };
}
exports.buildFetcherCreator = buildFetcherCreator;
