"use strict";
var chai_1 = require('chai');
var minimumLength_1 = require('./minimumLength');
describe('validator:minimumLength', function () {
    it('should return null for valid input', function () {
        var validator1 = minimumLength_1.minimumLength('A', 2, false);
        var validator2 = minimumLength_1.minimumLength('A', 3, true);
        chai_1.expect(validator1('12')).to.be.null;
        chai_1.expect(validator1('1234')).to.be.null;
        chai_1.expect(validator2('123')).to.be.null;
        chai_1.expect(validator2('123456')).to.be.null;
        chai_1.expect(validator2('')).to.be.null;
        chai_1.expect(validator2(null)).to.be.null;
        chai_1.expect(validator2(undefined)).to.be.null;
    });
    it('should return bundle with {VALUE, MINIMUM_LENGTH, ACCEPT_EMPTY} for invalid input', function () {
        var validator1 = minimumLength_1.minimumLength('A', 2, false);
        var validator2 = minimumLength_1.minimumLength('A', 3, true);
        chai_1.expect(validator1('1')).to.deep.equal({
            data: { ACCEPT_EMPTY: false, MINIMUM_LENGTH: 2, VALUE: '1' },
            key: 'A',
        });
        chai_1.expect(validator1('')).to.deep.equal({
            data: { ACCEPT_EMPTY: false, MINIMUM_LENGTH: 2, VALUE: '' },
            key: 'A',
        });
        chai_1.expect(validator1(null)).to.deep.equal({
            data: { ACCEPT_EMPTY: false, MINIMUM_LENGTH: 2, VALUE: null },
            key: 'A',
        });
        chai_1.expect(validator1(undefined)).to.deep.equal({
            data: { ACCEPT_EMPTY: false, MINIMUM_LENGTH: 2, VALUE: undefined },
            key: 'A',
        });
        chai_1.expect(validator2('1')).to.deep.equal({
            data: { ACCEPT_EMPTY: true, MINIMUM_LENGTH: 3, VALUE: '1' },
            key: 'A',
        });
        chai_1.expect(validator2('12')).to.deep.equal({
            data: { ACCEPT_EMPTY: true, MINIMUM_LENGTH: 3, VALUE: '12' },
            key: 'A',
        });
    });
});
