"use strict";
var chai_1 = require('chai');
var rangedNumber_1 = require('./rangedNumber');
describe('validator:rangedNumber', function () {
    it('should return null for valid input', function () {
        var validator1 = rangedNumber_1.rangedNumber('A', 1.5, 1.6);
        var validator2 = rangedNumber_1.rangedNumber('A', 1.5, 1.6, false, false);
        var validator3 = rangedNumber_1.rangedNumber('A', 3);
        chai_1.expect(validator1(1.5)).to.be.null;
        chai_1.expect(validator1(1.6)).to.be.null;
        chai_1.expect(validator2(1.51)).to.be.null;
        chai_1.expect(validator3(Number.POSITIVE_INFINITY)).to.be.null;
    });
    it('should return bundle with {VALUE, FROM, TO, INCLUSIVE_FROM, INCLUSIVE_TO} for invalid input', function () {
        var validator = rangedNumber_1.rangedNumber('A', -10, 10, true, false);
        chai_1.expect(validator(null)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: null,
            },
            key: 'A',
        });
        chai_1.expect(validator(undefined)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: undefined,
            },
            key: 'A',
        });
        chai_1.expect(validator(Number.NaN)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: Number.NaN,
            },
            key: 'A',
        });
        chai_1.expect(validator(Number.POSITIVE_INFINITY)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: Number.POSITIVE_INFINITY,
            },
            key: 'A',
        });
        chai_1.expect(validator(Number.NEGATIVE_INFINITY)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: Number.NEGATIVE_INFINITY,
            },
            key: 'A',
        });
        chai_1.expect(validator(-10.1)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: -10.1,
            },
            key: 'A',
        });
        chai_1.expect(validator(10)).to.deep.equal({
            data: {
                FROM: -10,
                INCLUSIVE_FROM: true,
                INCLUSIVE_TO: false,
                TO: 10,
                VALUE: 10,
            },
            key: 'A',
        });
    });
});
